#!/usr/bin/env Rscript
## ---------------------------
##
## Script name: process shapefiles
##
## Purpose of script: process shapefiles
## Author: Daniela Lührsen & Emily Ball
## Date Created: 2025-03-07
## Email: daniela.luhrsen@bsc.es
##
## ---------------------------

library(sf)
library(dplyr)
library(readr)
library(gridExtra)

# select parameters:
region <- "europe"
level <- "NUTS3"

get_shapes <- function(level) {
  if (level == "europe") {
    shp <- sf::st_read("data/WHO_Europe/new_who_europe.shp")
    shp <- shp[which(shp$country != "Russian Federation" &
                     shp$country != "Moldova" &
                     shp$country != "Ukraine" &
                     shp$country != "Andorra" &
                     shp$country != "Belarus" &
                     shp$erp_sb_ != "Central Asia"), ]
    shp <- shp[which(shp$country != "Armenia" &
                     shp$country != "Azerbaijan" &
                     shp$country != "Georgia" &
                     shp$country != "Israel"), ]

    shp <- shp %>% summarise(geometry = sf::st_union(geometry)) 
    shp$adm1_id <- "europe"
    shp$adm1_name <- "europe"
    sf::st_write(shp, dsn = "data/outputs", layer = "europe",
                 driver = "ESRI Shapefile", delete_layer = TRUE)

  } else if (level == "IS03") {
    adm3 <- sf::st_read("data/WHO_Europe/new_who_europe.shp")
    ISO2 <- adm3$ISO2
    ISO3 <- adm3$ISO3

    adm1 <- sf::st_read("data/NUTS_2024/NUTS_RG_01M_2024_3857.shp")
    adm1 <- adm1[which(adm1$CNTR_CODE %in% adm3$ISO2), ]
    # remove peripheral regions
    adm1 <- adm1[which(adm1$NUTS_ID != "FRY"   &
                         adm1$NUTS_ID != "FRY1"  &
                         adm1$NUTS_ID != "FRY2"  &
                         adm1$NUTS_ID != "FRY3"  &
                         adm1$NUTS_ID != "FRY4"  &
                         adm1$NUTS_ID != "FRY5"  &
                         adm1$NUTS_ID != "FRY10" &
                         adm1$NUTS_ID != "FRY20" &
                         adm1$NUTS_ID != "FRY30" &
                         adm1$NUTS_ID != "FRY40" &
                         adm1$NUTS_ID != "FRY50" &
                         adm1$NUTS_ID != "NO0B"  &
                         adm1$NUTS_ID != "NO0B1" &
                         adm1$NUTS_ID != "NO0B2"), ]
    adm1 <- adm1[adm1$LEVL_CODE == 3, ]
    adm1 <- adm1 %>%
      group_by(CNTR_CODE) %>%
      summarise(geometry = sf::st_union(geometry), .groups = "drop")

    adm2 <- sf::st_read(paste0("data/NUTS_2021/",
                               "NUTS_RG_01M_2021_3857_LEVL_3.shp"))
    adm2 <- adm2[which(adm2$CNTR_CODE == "UK"), ]
    adm2 <- sf::st_transform(adm2, sf::st_crs(adm1))
    adm2 <- adm2 %>%
      group_by(CNTR_CODE) %>%
      summarise(geometry = sf::st_union(geometry), .groups = "drop")

    adm3 <- adm3[which(adm3$ISO3 == "BIH"), ]
    adm3 <- sf::st_transform(adm3, sf::st_crs(adm1))

    colnames(adm1)[which(colnames(adm1) == "CNTR_CODE")] <- "adm1_id"
    colnames(adm2)[which(colnames(adm2) == "CNTR_CODE")] <- "adm1_id"

    adm3 <- adm3[, which(colnames(adm3) %in% c("ISO2",
                                               "geometry"))]
    colnames(adm3)[which(colnames(adm3) == "ISO2")] <- "adm1_id"

    # merge all dataframes
    shp <- rbind(adm1, adm2, adm3)
    ISO3 <- ISO3[which(ISO2 %in% shp$adm1_id)]
    

    sf::st_write(shp, dsn = "data/outputs", layer = "ISO3",
                 driver = "ESRI Shapefile", delete_layer = TRUE)

  } else if (level == "european_region") {
    shp <- sf::st_read("data/WHO_Europe/new_who_europe.shp")
    shp <- shp[which(shp$country != "Russian Federation" &
                     shp$country != "Moldova" &
                     shp$country != "Ukraine" &
                     shp$country != "Andorra" &
                     shp$country != "Belarus" &
                     shp$erp_sb_ != "Central Asia"), ]
    shp <- shp[which(shp$country != "Armenia" &
                     shp$country != "Azerbaijan" &
                     shp$country != "Georgia" &
                     shp$country != "Israel"), ]
    # group by European region
    shp <- shp %>%
      group_by(erp_sb_) %>%
      summarise(geometry = sf::st_union(geometry), .groups = "drop")

    # rename columns
    colnames(shp)[which(colnames(shp) == "erp_sb_")] <- "adm1_id"
    shp$adm1_name <- shp$adm1_id
    # save shp
    sf::st_write(shp, dsn = "data/outputs", layer = "european_region",
                 driver = "ESRI Shapefile", delete_layer = TRUE)

  } else if (level == "NUTS3") {
    adm1 <- sf::st_read("data/NUTS_2024/NUTS_RG_01M_2024_3857.shp")

    # remove peripheral regions
    adm1 <- adm1[which(adm1$NUTS_ID != "FRY"   &
                         adm1$NUTS_ID != "FRY1"  &
                         adm1$NUTS_ID != "FRY2"  &
                         adm1$NUTS_ID != "FRY3"  &
                         adm1$NUTS_ID != "FRY4"  &
                         adm1$NUTS_ID != "FRY5"  &
                         adm1$NUTS_ID != "FRY10" &
                         adm1$NUTS_ID != "FRY20" &
                         adm1$NUTS_ID != "FRY30" &
                         adm1$NUTS_ID != "FRY40" &
                         adm1$NUTS_ID != "FRY50" &
                         adm1$NUTS_ID != "NO0B"  &
                         adm1$NUTS_ID != "NO0B1" &
                         adm1$NUTS_ID != "NO0B2"), ]
    adm1 <- adm1[adm1$LEVL_CODE == 3, ]

    adm2 <- sf::st_read(paste0("data/NUTS_2021/",
                               "NUTS_RG_01M_2021_3857_LEVL_3.shp"))
    adm2 <- adm2[which(adm2$CNTR_CODE == "UK"), ]
    adm2 <- sf::st_transform(adm2, sf::st_crs(adm1))
    adm3 <- sf::st_read("data/WHO_Europe/new_who_europe.shp")
    adm3 <- adm3[which(adm3$ISO3 == "BIH"), ]
    adm3 <- sf::st_transform(adm3, sf::st_crs(adm1))

    adm1 <- adm1[, which(colnames(adm1) %in% c("NUTS_ID",
                                               "NUTS_NAME",
                                               "geometry"))]
    colnames(adm1)[which(colnames(adm1) == "NUTS_ID")] <- "adm1_id"
    colnames(adm1)[which(colnames(adm1) == "NUTS_NAME")] <- "adm1_name"

    adm2 <- adm2[, which(colnames(adm2) %in% c("NUTS_ID",
                                               "NUTS_NAME",
                                               "geometry"))]
    colnames(adm2)[which(colnames(adm2) == "NUTS_ID")] <- "adm1_id"
    colnames(adm2)[which(colnames(adm2) == "NUTS_NAME")] <- "adm1_name"

    adm3 <- adm3[, which(colnames(adm3) %in% c("ISO3",
                                               "country",
                                               "geometry"))]
    colnames(adm3)[which(colnames(adm3) == "ISO3")] <- "adm1_id"
    colnames(adm3)[which(colnames(adm3) == "country")] <- "adm1_name"

    # merge all dataframes
    shp <- rbind(adm1, adm2, adm3)

    # save shp
    sf::st_write(shp, dsn = "data/outputs", layer = "NUTS3",
                 driver = "ESRI Shapefile", delete_layer = TRUE)
  }
}